-- $Name:Омонимия$
-- $Version:1.2$
-- $Author:Luka$
-- $Info: Классический квест в мире слов, где каждое из них несет несколько значений.$

--    Омонимия/Homonymy
--    Copyright (C) 2024  Luka (ot-luki@yandex.ru)
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <https://www.gnu.org/licenses/>.


require "fmt" -- некоторые функции форматирования
require "noinv"
require "theme"
require "autotheme"
require 'fading' -- затенение

fmt.para = true -- включить режим параграфов (отступы)

game.act = 'Действие на предмет не прописано.';
game.use = 'Это не имеет смысла.';
game.inv = 'Описание предмета не задано.';

xact.walk = walk;

-- Переход с затенением. Отключать затенение приходится на выходе из локации
xact.fwalk = function (rm)
fading.change {'fadeblack', max = 64, delay = 20}
walk(rm)
end


function init()

	std.strip_call = false;

end

-- Изменения порядка отображения сообщений
--[[
function std.world:display(state)
		local l, av, pv
		local reaction = self:reaction() or nil
		if state then
			reaction = iface:em(reaction)
			av, pv = self:events()
			av = iface:em(av)
			pv = iface:em(pv)
			l = self.player:look() -- objects [and scene]
		end
		l = std.par(std.scene_delim, l or false, reaction or false,
			    av or false,
			    pv or false) or ''
		return l
end;
--]]

--std.scene_delim = '-'

function std.world:display(state)
local l, av, pv
local reaction = self:reaction() or nil
if state then
    reaction = iface:em(reaction)
    if reaction then reaction='^'..reaction end
    av, pv = self:events()
    av = iface:em(av)
    pv = iface:em(pv)
    l = self.player:look() -- objects [and scene]
    end
    l = std.par(std.scene_delim,
                reaction or false,
                av or false,
                l or false,
                pv or false) or ''
                return l
                end;

------------------------ Главное меню ----------------------------

room {
	nam = 'main';
	disp = 'Главное меню';
	noinv = true;
    --onenter = function(s,f) theme.gfx.bg('img/fonnoinv.png'); end;
    --onexit = function(s,f) theme.gfx.bg('img/fon.png'); end;
	pic = 'image/logo.png';
	decor = fmt.r([[
‘Вещь, имя получившая, тотчас^
становится немедля частью речи’.^
Иосиф Бродский]])..[[
^^
- {@ fwalk Вступление|Играть}^
- {@ walk От_автора|От автора}^
- {@ walk Помощь|Помощь}
]];
}


room{
	nam = 'От_автора';
    disp = 'От автора';
	noinv = true;
	way = {'main', 'Помощь'};
	decor = [[Автор идеи: Лука^
	Программирование, иллюстрации, текст, оформление темы: Лука^
    Игровой движок (Instead): Пётр Косых^
	^
	Пишите письма: ot-luki@yandex.ru^
    Исходный код игры распространяется по лицензии GNU GPL v3]];
	}
	
room{
	nam = 'Помощь';
	noinv = true;
	way = {'main', 'От_автора'};
	decor = [[
    -{$fmt nb| }Вещи не такие, какими кажутся.^
    -{$fmt nb| }Новые локации появляются при применении предметов на чистые листы бумаги.^
	-{$fmt nb| }В игре не предусмотрено летальных исходов или непроходимых ситуаций.^
    -{$fmt nb| }За редким исключением все отмеченные предметы можно взять в инвентарь, даже те, которые обычно поднять нельзя. Это не ошибка, так задумано.^
    -{$fmt nb| }Единовременно в инвентаре может быть не более двух предметов, не считая листов бумаги. Ненужные до времени предметы можно оставить в почти любой локации, не обязательно там, где они добыты. Для этого все локации имеют активную область "Оставить". Других функций у этой области нет.^
    -{$fmt nb| }Действие предметом на предмет не всегда работает одинаково в обе стороны, пользуйтесь здравым смыслом.^
    -{$fmt nb| }Правильные шаги решения, как правило, необратимы.^
    -{$fmt nb| }Если вам не нравится выравнивание основного текста по ширине, вы можете отключить его в настройках. Там же возможна корректировка размера шрифта.^
    -{$fmt nb| }Игра имеет отдельную тему для мобильных телефонов: Меню — Выбор темы — Омонимия. Мобильная.^
    ^
    Если вам нужна помощь по использованию интерфейса Instead, следует ознакомиться с поставляемым вместе с интерпретатором туториалом: Меню -- Выбор игры -- Обучение.^
    ^
    Краткое прохождение игры вы можете найти в файле solution.txt.
	]];
	}
	

----------- Автоматизация ----------------

-- Специальный объект позволяет выкинуть любой предмет на локации, где положен
obj {
nam = 'Оставить';
act = 'Вы можете оставить здесь свои предметы, чтобы взять позже.';
used = function(s,w)
        if w^'Бумага' then
            return false;
        else
            place(w);
        end;
    --dprint(s.my1());
    end;
}

-- Этот текст вставляется в начало локации для того, чтобы создать в ней отдельную "кнопку" для сброса предметов
global {
    dropButton = fmt.r('^')..fmt.r('[{Оставить|Оставить}]^')..fmt.middle('^')
}

objj = std.class({
    __objj_type = true;
    dsc = function(s)
        if not s.home or not s.dropped then return ''; end;
        if where(s) ^ s.home  then return '';
        elseif where(s)^'Корабль' then return ''; end;
        for i=1, #s.home do
            if where(s) ^ s.home[i]  then return ''; end;
        end;
        return(s.dropped)
    end;
}, std.obj)

    
-- Возвращает текст, только если условие истинно. Иначе - ''
ifp = function(cond, str)
    if cond then return str;
    else return '';
    end;
end;

-- Эта функция дает поднять предмет только если руки свободны
mytake = function(s, tak)
    if #inv()<3 then
        p(tak);
        return true;
    else
        p 'Вы не можете таскать больше двух предметов за раз.';
        return false;
    end;
end;

-- Объект-счетчик пустых листов бумаги
obj {
nam = 'Бумага';
--disp = function(s) return 'Чистых листов: '.. s.n ..'^^'; end;
disp = function(s)
        if theme.name() == '.default' then -- наша встроенная тема default
            return 'Чистые листы^'
        else
            return 'Чистые листы'
        end -- if
        end; -- func
n = 1000;
inv = 'Чистые листы бумаги.';
dsc = 'Вы оставили здесь чистые {листы бумаги}.';
tak = 'Вы подняли листы.';
use = function(s,w)
        if w^'Оставить' then p "Лучше держать их при себе.";
        elseif w^'Вода' then p 'Потом их не найти.';
        elseif w^'ОставитьБ' then p 'Потом их не найти.';
        elseif w^'ОставитьЛавка' then p 'Это не годится в уплату.';
        elseif w^'ОставитьБор' then p 'Лучше держать их при себе.';
        elseif w^'Личинка' then p 'Ни к чему заворачивать это в бумагу.';
        elseif w^'Сарай' then p 'Не попробовать ли наоборот?';
        elseif w^'ОДвор' then p 'Не попробовать ли наоборот?';
        elseif w^'Ключ' then p 'Они же намокнут!';
        elseif w^'Обрыв' then p 'Не будем мусорить.';
        elseif w^'Бот' then p 'Тут бумажкой не обойтись.';
        elseif w^'Стакан' then p 'Он и так чистый.';
        elseif w^'Гвоздика' then p 'Уродливый вышел бы букет.';
        elseif w^'Лавка' then p 'Подстелить было бы не плохо, но не время просиживать штаны.';
        elseif w^'Собачка' then p 'Она не будет это есть.';
        elseif w^'Собачка2' then p 'Она не будет это есть.';
        elseif w^'Верстак' then p 'С этим не нужно ничего делать.';
        elseif w^'Карабин' then p 'Неплохая идея, если вам понадобится пыж.';
        else p 'Ничего не получается.';
        end;
        return true;
        end;
}

-- Функция добавляет локацию во всех проходы всех комнат
addSite = function(rm)
    std.for_each_obj(function(v)
                    if v:type 'room' then
                        -- dprint(v.nam);
                        v.way:add(rm);
                        end
                    end)
end;

-- Функция убирает локацию из всех проходы всех комнат
delSite = function(rm)
    std.for_each_obj(function(v)
                    if v:type 'room' then
                        -- dprint(v.nam);
                        v.way:del(rm);
                        end
                    end)
end;

-- Если thing находится в rm, возвращает ее .inroomdesc, иначе outroom
deskr = function(rm,thing)
    if where(_(thing)) ^ rm then return _(thing).inroomdesc
    else return _(thing).outroomdesc
    end;
end;


room {
nam = 'Вступление';
pic = 'image/konvert.png';
noinv = true;
    --onenter = function(s,f) theme.gfx.bg('img/fonnoinv.png'); end;
decor = [[Незапечатанный конверт. Это все что осталось...
^^
{@ fwalk Письмо|<Прочесть письмо>}]];

onexit = function(s,t)
    -- fading.change{'none'}
    --theme.gfx.bg('img/fon.png');
    take 'Бумага';
    addSite('Письмо');
    addSite('Изба');
    --addSite('Двор');
    --addSite('Лес');
    end;
}

include 'act1.lua'

include 'act2.lua'

include 'act3.lua'
